@@ -48,6 +48,9 @@ struct l_settings *network_get_settings(const struct network *network);
 bool network_set_psk(struct network *network, const uint8_t *psk);
 void network_sync_psk(struct network *network);
 
+const struct network_info *network_get_info(const struct network *network);
+void network_set_info(struct network *network, struct network_info *info);
+
 int network_autoconnect(struct network *network, struct scan_bss *bss);
 void network_connect_failed(struct network *network);
 bool network_bss_add(struct network *network, struct scan_bss *bss);
@@ -69,19 +72,3 @@ void network_connect_new_hidden_network(struct network *network,
 						struct l_dbus_message *message);
 
 void network_blacklist_add(struct network *network, struct scan_bss *bss);
-
-struct network_info {
-	char ssid[33];
-	enum security type;
-	struct timespec connected_time;	/* Time last connected */
-	int seen_count;			/* Ref count for network.info */
-	bool is_hidden:1;
-	bool is_known:1;
-	bool is_autoconnectable:1;
-};
-
-struct network_info *network_info_add_known(const char *ssid,
-						enum security security);
-void network_info_forget_known(struct network_info *network);
-bool network_info_match(const void *a, const void *b);
-void network_info_free(void *data);
