@@ -80,12 +80,13 @@ static bool enable_debug;
 const char *debug_filter;
 static struct l_settings *hw_config;
 static bool native_hw;
+static bool shell;
 static const char *qemu_binary;
 static const char *kernel_image;
 static const char *exec_home;
 static const char *test_action_params;
 static char top_level_path[PATH_MAX];
-struct l_queue *wiphy_list;
+static struct l_queue *wiphy_list;
 
 #if defined(__i386__)
 /*
@@ -386,7 +387,7 @@ static bool start_qemu(void)
 			"TESTVERBOUT=\'%s\' DEBUG_FILTER=\'%s\'"
 			"TEST_ACTION=%u TEST_ACTION_PARAMS=\'%s\' "
 			"TESTARGS=\'%s\' PATH=\'%s\' VALGRIND=%u"
-			"GDB=\'%s\' HW=\'%s\'",
+			"GDB=\'%s\' HW=\'%s\' SHELL=%u",
 			check_verbosity("kernel") ? "ignore_loglevel" : "quiet",
 			initcmd, cwd, verbose_opt ? verbose_opt : "none",
 			enable_debug ? debug_filter : "",
@@ -396,7 +397,8 @@ static bool start_qemu(void)
 			getenv("PATH"),
 			valgrind,
 			gdb_opt ? gdb_opt : "none",
-			hw_config ? "real" : "virtual");
+			hw_config ? "real" : "virtual",
+			shell);
 
 	if (hw_config) {
 		if (l_settings_has_group(hw_config, "PCIAdapters")) {
@@ -488,7 +490,7 @@ static bool start_qemu(void)
 	return true;
 }
 
-static pid_t execute_program(char *argv[], bool wait, bool verbose)
+static pid_t execute_program(char *argv[], bool wait, bool verbose, bool log)
 {
 	int status;
 	pid_t pid, child_pid;
@@ -509,7 +511,16 @@ static pid_t execute_program(char *argv[], bool wait, bool verbose)
 
 	if (child_pid == 0) {
 		if (!verbose) {
-			int fd = open("/dev/null", O_WRONLY);
+			int fd;
+			char log_file[PATH_MAX];
+
+			if (!log)
+				fd = open("/dev/null", O_WRONLY);
+			else {
+				sprintf(log_file, "/tmp/%s.log", argv[0]);
+				fd = open(log_file, O_WRONLY | O_CREAT,
+							S_IRUSR | S_IWUSR);
+			}
 
 			dup2(fd, 1);
 			dup2(fd, 2);
@@ -615,7 +626,7 @@ static bool start_dbus_daemon(void)
 	if (check_verbosity("dbus"))
 		setenv("DBUS_VERBOSE", "1", true);
 
-	pid = execute_program(argv, false, check_verbosity("dbus"));
+	pid = execute_program(argv, false, check_verbosity("dbus"), false);
 	if (pid < 0)
 		return false;
 
@@ -626,7 +637,7 @@ static bool start_dbus_daemon(void)
 		argv[0] = "dbus-monitor";
 		argv[1] = "--system";
 		argv[2] = NULL;
-		execute_program(argv, false, true);
+		execute_program(argv, false, true, false);
 	}
 
 	l_debug("D-Bus is running");
@@ -642,7 +653,7 @@ static bool start_haveged(void)
 	argv[0] = "haveged";
 	argv[1] = NULL;
 
-	pid = execute_program(argv, true, false);
+	pid = execute_program(argv, true, false, false);
 	if (pid < 0)
 		return false;
 
@@ -664,7 +675,7 @@ static bool set_interface_state(const char *if_name, bool isUp)
 	argv[2] = state;
 	argv[3] = NULL;
 
-	pid = execute_program(argv, true, false);
+	pid = execute_program(argv, true, false, false);
 	if (pid < 0)
 		return false;
 
@@ -686,7 +697,7 @@ static bool create_interface(const char *if_name, const char *phy_name)
 	argv[7] = "managed";
 	argv[8] = NULL;
 
-	pid = execute_program(argv, true, false);
+	pid = execute_program(argv, true, false, false);
 	if (pid < 0)
 		return false;
 
@@ -704,7 +715,7 @@ static bool delete_interface(const char *if_name)
 	argv[3] = "del";
 	argv[4] = NULL;
 
-	pid = execute_program(argv, true, false);
+	pid = execute_program(argv, true, false, false);
 	if (pid < 0)
 		return false;
 
@@ -720,7 +731,7 @@ static bool list_interfaces(void)
 	argv[1] = "-a";
 	argv[2] = NULL;
 
-	pid = execute_program(argv, true, true);
+	pid = execute_program(argv, true, true, false);
 	if (pid < 0)
 		return false;
 
@@ -736,7 +747,7 @@ static bool list_hwsim_radios(void)
 	argv[1] = "--list";
 	argv[2] = NULL;
 
-	pid = execute_program(argv, true, true);
+	pid = execute_program(argv, true, true, false);
 	if (pid < 0)
 		return false;
 
@@ -784,7 +795,7 @@ static int create_hwsim_radio(const char *radio_name,
 
 	argv[idx] = NULL;
 
-	pid = execute_program(argv, true, check_verbosity(BIN_HWSIM));
+	pid = execute_program(argv, true, check_verbosity(BIN_HWSIM), false);
 	if (pid < 0)
 		return -1;
 
@@ -803,7 +814,7 @@ static bool destroy_hwsim_radio(int radio_id)
 	argv[1] = destroy_param;
 	argv[2] = NULL;
 
-	pid = execute_program(argv, true, false);
+	pid = execute_program(argv, true, false, false);
 	if (pid < 0)
 		return false;
 
@@ -823,7 +834,7 @@ static pid_t register_hwsim_as_trans_medium(void)
 	argv[idx++] = BIN_HWSIM;
 	argv[idx++] = NULL;
 
-	return execute_program(argv, false, check_verbosity(BIN_HWSIM));
+	return execute_program(argv, false, check_verbosity(BIN_HWSIM), false);
 }
 
 static void terminate_medium(pid_t medium_pid)
@@ -847,13 +858,13 @@ static void start_loopback(void)
 	argv[2] = "127.0.0.1";
 	argv[3] = "up";
 	argv[4] = NULL;
-	execute_program(argv, false, false);
+	execute_program(argv, false, false, false);
 
 	argv[0] = "route";
 	argv[1] = "add";
 	argv[2] = "127.0.0.1";
 	argv[3] = NULL;
-	execute_program(argv, false, false);
+	execute_program(argv, false, false, false);
 
 	loopback_started = true;
 }
@@ -872,7 +883,7 @@ static pid_t start_phonesim(void)
 
 	setenv("OFONO_PHONESIM_CONFIG", "/tmp/phonesim.conf", true);
 
-	return execute_program(argv, false, false);
+	return execute_program(argv, false, false, false);
 }
 
 static void stop_phonesim(pid_t pid)
@@ -898,7 +909,7 @@ static pid_t start_ofono(void)
 
 	start_loopback();
 
-	return execute_program(argv, false, verbose);
+	return execute_program(argv, false, verbose, false);
 }
 
 static void stop_ofono(pid_t pid)
@@ -953,7 +964,7 @@ static pid_t start_hostapd(char **config_files, struct wiphy **wiphys)
 		argv[idx++] = NULL;
 	}
 
-	pid = execute_program(argv, false, verbose);
+	pid = execute_program(argv, false, verbose, false);
 	if (pid < 0) {
 		goto exit;
 	}
@@ -1402,7 +1413,7 @@ static pid_t start_iwd(const char *config_dir, struct l_queue *wiphy_list,
 	argv[idx++] = "-c";
 	argv[idx++] = (char *) config_dir;
 
-	if (check_verbosity(BIN_IWD))
+	if (check_verbosity(BIN_IWD) || shell)
 		argv[idx++] = "-d";
 
 	argv[idx] = NULL;
@@ -1443,7 +1454,7 @@ static pid_t start_iwd(const char *config_dir, struct l_queue *wiphy_list,
 	argv[idx++] = (char *)ext_options;
 	argv[idx] = NULL;
 
-	ret = execute_program(argv, false, check_verbosity(BIN_IWD));
+	ret = execute_program(argv, false, check_verbosity(BIN_IWD), shell);
 
 	if (iwd_phys)
 		l_free(iwd_phys);
@@ -1698,7 +1709,7 @@ start_next_test:
 
 	print_test_status(py_test, TEST_STATUS_STARTED, 0);
 	test_exec_pid = execute_program(argv, false,
-			check_verbosity("pytests"));
+			check_verbosity("pytests"), false);
 
 	gettimeofday(&time_before, NULL);
 
@@ -1839,7 +1850,7 @@ static void set_reg_domain(const char *domain)
 	argv[3] = (char *) domain;
 	argv[4] = NULL;
 
-	execute_program(argv, false, false);
+	execute_program(argv, false, false, false);
 }
 
 static void wiphy_up(void *data, void *user_data)
@@ -2045,7 +2056,12 @@ static void create_network_and_run_tests(const void *key, void *value,
 
 	set_wiphy_list(wiphy_list);
 
-	run_py_tests(hw_settings, test_queue, test_stats_queue);
+	if (!shell)
+		run_py_tests(hw_settings, test_queue, test_stats_queue);
+	else {
+		if (system("/bin/sh"))
+			l_info("executing /bin/sh failed");
+	}
 
 	l_info("Destructing network...");
 
@@ -2264,8 +2280,10 @@ static void run_auto_tests(void)
 	if (!start_dbus_daemon())
 		goto exit;
 
-	if (!start_haveged())
+	if (!start_haveged()) {
+		l_error("Failed to start haveged");
 		goto exit;
+	}
 
 	test_stat_queue = l_queue_new();
 
@@ -2303,7 +2321,7 @@ static void run_unit_tests(void)
 		argv[0] = unit_test_abs_path;
 		argv[1] = NULL;
 
-		execute_program(argv, true, check_verbosity("unit"));
+		execute_program(argv, true, check_verbosity("unit"), false);
 
 		l_free(unit_test_abs_path);
 	}
@@ -2551,6 +2569,14 @@ static void run_tests(void)
 		return;
 	}
 
+	ptr = strstr(cmdline, "SHELL=");
+	if (ptr) {
+		*ptr = '\0';
+		test_action_str = ptr + 6;
+
+		shell = atoi(test_action_str);
+	}
+
 	ptr = strstr(cmdline, "HW=");
 	if (ptr) {
 		*ptr = '\0';
@@ -2742,7 +2768,8 @@ static void usage(void)
 		"\t-g, --gdb <iwd|hostapd>	Run gdb on the specified"
 						" executable\n"
 		"\t-w, --hw <config>	Run using a physical hardware "
-					"configuration");
+					"configuration\n"
+		"\t-s, --shell		Boot into shell, not autotests\n");
 	l_info("Commands:\n"
 		"\t-A, --auto-tests <dirs>	Comma separated list of the "
 						"test configuration\n\t\t\t\t"
@@ -2761,6 +2788,7 @@ static const struct option main_options[] = {
 	{ "gdb",	required_argument, NULL, 'g' },
 	{ "valgrind",	no_argument,       NULL, 'V' },
 	{ "hw",		required_argument, NULL, 'w' },
+	{ "shell",	optional_argument, NULL, 's' },
 	{ "help",	no_argument,       NULL, 'h' },
 	{ }
 };
@@ -2847,6 +2875,9 @@ int main(int argc, char *argv[])
 				return EXIT_FAILURE;
 			}
 			break;
+		case 's':
+			shell = true;
+			break;
 		case 'h':
 			usage();
 			return EXIT_SUCCESS;
